@@ -99,23 +99,13 @@ class Utilities(web.Utilities):
         if rv is not None:
             return rv
 
-        rv = obj.getRole() == pyatspi.ROLE_STATIC and not self._getTag(obj) \
-            and obj.parent.getRole() == pyatspi.ROLE_LIST_ITEM \
-            and obj.getIndexInParent() == 0
+        rv = False
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_ITEM:
+            rv = self._getTag(obj) in ["::marker", None] and obj.parent[0] == obj
 
         self._isListItemMarker[hash(obj)] = rv
         return rv
 
-    def getListItemMarkerText(self, obj):
-        if obj.getRole() != pyatspi.ROLE_LIST_ITEM:
-            return ""
-
-        for child in obj:
-            if self.isListItemMarkerInSimpleItem(child):
-                return child.name
-
-        return ""
-
     def selectedChildCount(self, obj):
         count = super().selectedChildCount(obj)
         if count or "Selection" in pyatspi.listInterfaces(obj):
@@ -244,6 +234,12 @@ class Utilities(web.Utilities):
 
         return False
 
+    def isTopLevelMenu(self, obj):
+        if obj.getRole() == pyatspi.ROLE_MENU:
+            return self.isFrameForPopupMenu(self.topLevelObject(obj))
+
+        return False
+
     def popupMenuForFrame(self, obj):
         if not self.isFrameForPopupMenu(obj):
             return None
@@ -431,11 +427,13 @@ class Utilities(web.Utilities):
         if len(self.findAllDescendants(obj, isButton)) != 3:
             msg = "CHROMIUM: %s not believed to be find-in-page container (button count)" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
+            return False
 
         isSeparator = lambda x: x.getRole() == pyatspi.ROLE_SEPARATOR
         if len(self.findAllDescendants(obj, isSeparator)) != 1:
             msg = "CHROMIUM: %s not believed to be find-in-page container (separator count)" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
+            return False
 
         msg = "CHROMIUM: %s believed to be find-in-page container (accessibility tree)" % obj
         debug.println(debug.LEVEL_INFO, msg, True)
@@ -471,3 +469,55 @@ class Utilities(web.Utilities):
 
     def supportsLandmarkRole(self):
         return True
+
+    def findAllDescendants(self, root, includeIf=None, excludeIf=None):
+        if not root:
+            return []
+
+        # Don't bother if the root is a 'pre' or 'code' element. Those often have
+        # nothing but a TON of static text leaf nodes, which we want to ignore.
+        if self._getTag(root) in ('pre', 'code'):
+            msg = "CHROMIUM: Returning 0 descendants for pre/code %s" % root
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return []
+
+        return super().findAllDescendants(root, includeIf, excludeIf)
+
+    def _accessibleAtPoint(self, root, x, y, coordType=None):
+        if self.isHidden(root):
+            return None
+
+        try:
+            component = root.queryComponent()
+        except:
+            msg = "CHROMIUM: Exception querying component of %s" % root
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return None
+
+        result = component.getAccessibleAtPoint(x, y, coordType)
+
+        # Chromium cannot do a hit test of web content synchronously. So what it
+        # does is return a guess, then fire off an async hit test. The next time
+        # one calls it, Chromium returns the previous async hit test result if
+        # the point is still within its bounds. Therefore, we need to call
+        # getAccessibleAtPoint() twice to be safe.
+        result = component.getAccessibleAtPoint(x, y, coordType)
+
+        msg = "CHROMIUM: %s is descendant of %s at (%i, %i)" % (result, root, x, y)
+        debug.println(debug.LEVEL_INFO, msg, True)
+        return result
+
+    def descendantAtPoint(self, root, x, y, coordType=None):
+        if coordType is None:
+            coordType = pyatspi.DESKTOP_COORDS
+
+        result = None
+        if self.isDocument(root):
+            result = self._accessibleAtPoint(root, x, y, coordType)
+
+        root = result or root
+        result = super().descendantAtPoint(root, x, y, coordType)
+        if self.isListItemMarker(result) or self.isStaticTextLeaf(result):
+            return result.parent
+
+        return result
